from django.core.management.base import BaseCommand

from ...config import configure_publishers


class Command(BaseCommand):
    help = 'Create all publishers to can consume messages via postoffice'

    def handle(self, *args, **kwargs):
        configure_publishers()



# Generated by Django 3.0.2 on 2020-01-15 11:51

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='PublishingError',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('topic', models.CharField(max_length=100)),
                ('payload', django.contrib.postgres.fields.jsonb.JSONField()),
                ('attributes', django.contrib.postgres.fields.jsonb.JSONField(null=True)),
                ('error', models.CharField(default='', max_length=255)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
            ],
        ),
    ]



import logging
from http import HTTPStatus

import requests

from . import settings
from postoffice_django.exceptions import (BadPublisherCreation,
    BadTopicCreation)
from requests.exceptions import ConnectTimeout, ConnectionError


logger = logging.getLogger(__name__)


def configure_publishers() -> None:
    uncreated_publishers = []

    for consumer in settings.get_consumers():
        if not _create_publishers(consumer):
            uncreated_publishers.append(consumer)

    if uncreated_publishers:
        raise BadPublisherCreation(uncreated_publishers)


def configure_topics() -> None:
    uncreated_topics = []

    for topic in settings.get_topics():
        if not _create_topic(topic):
            uncreated_topics.append(topic)

    if uncreated_topics:
        raise BadTopicCreation(uncreated_topics)


def _create_publishers(consumer: dict) -> None:
    url = f'{settings.get_url()}/api/publishers/'
    payload = {
        'active': True,
        'topic': consumer.get('topic'),
        'target': consumer.get('target'),
        'type': consumer.get('type'),
        'from_now': True
    }

    return _execute_request(url, payload)


def _create_topic(topic_name: str) -> None:
    url = f'{settings.get_url()}/api/topics/'
    payload = {'name': topic_name, 'origin_host': settings.get_origin_host()}

    return _execute_request(url, payload)


def _execute_request(url: str, payload: dict):
    try:
        response = requests.post(
            url,
            json=payload,
            timeout=settings.get_timeout()
        )
    except (ConnectTimeout, ConnectionError):
        return False

    return response.status_code == HTTPStatus.CREATED



import requests
from requests import Response
from requests.exceptions import ConnectionError, ConnectTimeout

from . import settings
from .models import PublishingError

CONNECTION_ERROR = 'Can not establish connection with postoffice'


def publish(topic: str, payload: dict, **attrs: dict) -> None:
    url = f'{settings.get_url()}/api/messages/'
    message = {
        'topic': topic,
        'payload': payload,
        'attributes': _stringify_attributes(attrs)
    }

    try:
        response = requests.post(url,
                                 json=message,
                                 timeout=settings.get_timeout()
                                 )
    except (ConnectTimeout, ConnectionError):
        _save_connection_not_stablished(message)
        return

    if response.status_code != 201:
        _save_publishing_error(response, message)


def _stringify_attributes(attributes: dict) -> dict:
    return {key: str(attributes[key]) for key in attributes.keys()}


def _save_connection_not_stablished(message: dict) -> None:
    _create_publishing_error(message,
                             CONNECTION_ERROR)


def _save_publishing_error(response: Response, message: dict) -> None:
    error = 'Uknown error'

    if response.status_code == 500:
        error = 'Internal server error'

    if response.status_code == 400:
        error = response.json().get('data').get('errors')

    _create_publishing_error(message, error)


def _create_publishing_error(message: dict, error: str) -> None:
    PublishingError.objects.create(
        topic=message.get('topic'),
        payload=message.get('payload'),
        attributes=message.get('attributes'),
        error=error,
    )



from django.contrib import admin

from .models import PublishingError


@admin.register(PublishingError)
class PublishingErrorAdmin(admin.ModelAdmin):

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def has_change_permission(self, request, obj=None):
        return False
